
#include "net_info_keeper.h"
#include "../timespec_utils.h"

void net_info_keeper_update_rtt(struct net_info_keeper *this,
				struct timespec *time_received,
				struct timespec *time_sent)
{

	const unsigned int AVERAGE_GAIN_RECIPROCAL = 3;
	const unsigned int DEVIATION_GAIN_RECIPROCAL = 2;	

	struct timespec rtt = { .tv_sec = 0, .tv_nsec = 0, };	
	s64 rtt_nsec = 0;
	s64 rtt_error;	
		
	if(timespec_compare(time_received, time_sent) >= 0){		
		rtt = timespec_sub( *time_received,
				    *time_sent );
		rtt_nsec = timespec_to_ns( &rtt );
	}			
	
	/*
	  rtts bigger than two seconds are usually caused by SYN-ACK resends
	  from the target and aren't rtts but rather the rtt + the
	  amount of time the target chose to wait until it resends the
	  SYN-ACK.
	 
	  We want to ignore these responses for timeout-calculation.
 
	 */

	if(rtt.tv_sec >= 2){
		printk("rtt bigger than 2 seconds encountered, discarding\n");
		return;
	}
				
	/* calculate new average round-trip-time ('rtt') */
	
	if(this->rtt_average == -1)
		this->rtt_average = 0;
	
	
	rtt_error = rtt_nsec - (this->rtt_average >> AVERAGE_GAIN_RECIPROCAL);					
	this->rtt_average += rtt_error;
	
	/* calculate new trigger-deviation */
	
	if(rtt_error < 0)
		rtt_error = -rtt_error;
	
	rtt_error -= (this->rtt_deviation >> DEVIATION_GAIN_RECIPROCAL);
	this->rtt_deviation += rtt_error;	
		       	

}


struct net_info_keeper *create_net_info_keeper(void)
{
	struct net_info_keeper *this = 
		kmalloc(sizeof(struct net_info_keeper), GFP_KERNEL);

	if(!this)
		return NULL;
	
	memset(this, 0, sizeof(struct net_info_keeper));
	
	this->rtt_average = -1;

	return this;
}

void delete_net_info_keeper(struct net_info_keeper *this)
{
	
	kfree(this);

}

void net_info_keeper_on_probe_rcv(struct net_info_keeper *this,
				  struct packet_batch *batch,
				  struct sniffed_packet_descr_t *pdescr)
{
	
}

void net_info_keeper_on_sent(struct net_info_keeper *this,
			     struct packet_batch *batch)
{
	/* update the number of active packets */			
	
	this->nactive_packets += (batch->size_of_batch + 1);

	/* update the number of packets sent */
	
	this->sent_counter += (batch->size_of_batch + 1);

}


void net_info_keeper_on_trigger_rcv(struct net_info_keeper *this,
				    struct packet_batch *batch)
{
	/* update the number of active packets */			
	this->nactive_packets -= (batch->size_of_batch + 1);
	
	this->ack_counter += (batch->size_of_batch + 1);

}

void net_info_keeper_on_timeout(struct net_info_keeper *this,
				struct packet_batch *batch)
{
	/* update the number of active packets */
	this->nactive_packets -= (batch->size_of_batch + 1);
}

