#ifndef _NET_INFO_KEEPER_H
#define _NET_INFO_KEEPER_H

#include "packet_batch.h"

/*
  Keeps information about the network's
  performance based on all events handed
  to it by the scan-job-manager.
  
  This information will then be processed
  by the timing-algorithm to change
  the timing-context which is the set of 
  timing-rules used to perform the scan.
  
*/

struct net_info_keeper{
		
	s64 rtt_average;
	s64 rtt_deviation;	
	u32 nactive_packets;
	
	/* contains the number of packets sent */
	u64 sent_counter;	
	/* contains the number of packets acknowledged */
	u64 ack_counter;

};


struct net_info_keeper *create_net_info_keeper(void);
void delete_net_info_keeper(struct net_info_keeper *this);

void net_info_keeper_on_probe_rcv(struct net_info_keeper *this,
				  struct packet_batch *batch,
				  struct sniffed_packet_descr_t *pdescr);

void net_info_keeper_on_sent(struct net_info_keeper *this,
			     struct packet_batch *batch);

void net_info_keeper_on_trigger_rcv(struct net_info_keeper *this,
				    struct packet_batch *batch);

void net_info_keeper_on_timeout(struct net_info_keeper *this,
				struct packet_batch *batch);

void net_info_keeper_update_rtt(struct net_info_keeper *this,
				struct timespec *time_received,
				struct timespec *time_sent);
	
#endif
