#ifndef _RECURITY_SCAN_JOB_MANAGER_H
#define _RECURITY_SCAN_JOB_MANAGER_H

#include <linux/wait.h>

/**
   \addtogroup ScanJobManager

   ScanJobManager

   The ScanJobManager is the central component
   of the system. It processes user-input, newly
   arrived packets and submitter-reports and
   forwards them to the scan-job in question if
   necessary. See the kernel thread's main function
   \ref scan_job_manager for details.

   @{

*/

/**
   @name Public functions
   @{

*/

/**
   Initializes the scan-job-manager.
   
   (1) Initializes all scan-job-states.   
   (2) Initializes command-handlers  
   (3) Initializes scan_jobs_hash   
   (4) Starts the scan-job-manager kernel-thread.
   
*/

int scan_job_manager_init(void);

/**
   deinitializes everything which was initialized
   In scan_job_manager_init
*/

void scan_job_manager_fini(void);



void scan_job_manager_wait_for_termination(void);

/** @}*/


/** wait-queue used to allow scan-job-manager sleeping
 when idle.
*/

extern wait_queue_head_t scan_job_man_wait;

extern struct task_struct *scan_job_man_thread;

/**
   @}

*/

#endif
