#!/usr/bin/env python

# Given a scan-log, plots the "per-trigger scan-throughput graph"

import Gnuplot, Gnuplot.funcutils
import sys

# Make sure user has supplied scan-log-filename and an ip

if len(sys.argv) != 3:
    print "Usage: " + sys.argv[0] + " <scan_log_file> <target_ip>"
    sys.exit(1)

scan_log_fname = sys.argv[1]
target_ip = sys.argv[2]

try:
    scan_log_file = open(scan_log_fname)
except:
    print "Error opening scan-log-file"
    sys.exit(1)

# Create gnuplot-object

g = Gnuplot.Gnuplot()
g.title("PortBunny per-trigger scan-throughput-graph")
g.ylabel('Results/s')
g.xlabel('Time [ns]')
g('set data style lines')


# Extract cwnds from scan-log-file

tput_dict = dict()
plot_tuple = ()
start_time_ns = 0

delta_ns = 50000000
frame_beg_ns = 0
nresults = 0

for line in scan_log_file:
    args = line.split(' ')
    
    if args[0] == 'INFO' and args[1] == 'TRIGGER_RECEIVED':
        
        # process only triggers for this host
        if args[2] != target_ip:
            continue
        
        trigger_name = args[4] + args[5]

        if not tput_dict.has_key(trigger_name):
            tput_dict[trigger_name] = []            

        if start_time_ns == 0:
            start_time_ns = int(args[6])
            frame_beg_ns = start_time_ns

        cur_time = int(args[6])
        
        if cur_time - frame_beg_ns >= delta_ns:            

            tput_dict[trigger_name].append([cur_time - start_time_ns, ( float(nresults + 1) * 10**9 ) /(cur_time - frame_beg_ns)  ])                    
            frame_beg_ns = cur_time
            nresults = 0
        else:
            nresults = nresults + 1
        
        
for trigger_name in tput_dict.keys():
    if tput_dict[trigger_name] != []:
        plot_tuple = plot_tuple + (Gnuplot.PlotItems.Data(tput_dict[trigger_name], title=trigger_name + "-scan-throughput"),)
    

g.plot(*(plot_tuple))
raw_input('press any  key')
